# vim:fileencoding=utf-8

import re
import string

from .strutils import force_unicode

__all__ = (
    'escape',
    'escape_entities',
    'sanitize_html',
    'urlize',

    'URL_VALID_CHARS',
    'URL_PATH_VALID_CHARS',
    'URL_QUERY_VALID_CHARS',
    'URL_FRAGMENT_VALID_CHARS',

    'DOMAIN_RE',
    'PORT_RE',
    'IP_ADDRESS_RE',
    'IP_PORT_RE',
    'IP_DOMAIN_RE',
    'URL_DOMAIN_RE',
    'URL_RE',
    'URL_RE_CMP',

    'LOOSE_DOMAIN_RE',
    'LOOSE_PORT_RE',
    'LOOSE_URL_RE',
    'LOOSE_URL_RE_CMP',
)

# Monkey-patch HTMLParser to allow parsing of html
# attributes whose values have no quotes and are non-ascii
# i.e. <input name=submit type=submit value=検索>
import HTMLParser
try:
    _p = HTMLParser.HTMLParser()
    _p.feed(u"<input name=submit type=submit value=検索>")
    _p.close()
except HTMLParser.HTMLParseError:
    # Only patch HTMLParser if it's needed.
    HTMLParser.attrfind = re.compile(
        r'\s*([a-zA-Z_][-.:a-zA-Z_0-9]*)(\s*=\s*'
        r'(\'[^\']*\'|"[^"]*"|[^">\s]*))?')


def escape(html):
    """
    Returns the given HTML with ampersands, quotes and angle brackets encoded.
    """
    return (force_unicode(html).replace('&', '&amp;')
                               .replace('<', '&lt;')
                               .replace('>', '&gt;')
                               .replace('"', '&quot;')
                               .replace("'", '&#39;'))


def escape_entities(text):
    return (re.sub(r'&(?!([A-Za-z]+)|(#[0-9]+);)', '&amp;', text)
              .replace('<', '&lt;')
              .replace('>', '&gt;')
              .replace('"', '&quot;')
              .replace("'", '&apos;'))

DEFAULT_VALID_TAGS = {
    'b': (),
    'blockquote': ('style',),
    'em': (),
    'strong': (),
    'strike': (),
    'a': ('href', 'title', 'rel'),
    'i': (),
    'br': (),
    'ul': (),
    'ol': (),
    'li': (),
    'u': (),
    'p': (),
    'h1': (),
    'h2': (),
    'h3': (),
    'h4': (),
    'table': (),
    'thead': (),
    'tbody': (),
    'tfoot': (),
    'th': (),
    'td': ('colspan',),
    'tr': ('rowspan',),
    'hr': (),
    'img': ('src', 'alt', 'title', 'width', 'height', 'align'),
    'span': ('style',),
    'div': ('style',),
    'font': ('size', 'style', 'color'),
}

DEFAULT_VALID_STYLES = (
    "background-color",
    "color",
    "margin",
    "margin-left",
    "margin-right",
    "border",
    "padding",
    "font-weight",
    "font-style",
    "font-size",
    "text-align",
    "text-decoration",
)

HTTP_SCHEME_RE = 'http[s]*'

# See: http://www.ietf.org/rfc/rfc1738.txt
URL_SAFE = "$-_.+"
URL_EXTRA = "!*'(),"
URL_PATH_RESERVED = ';?'
URL_QUERY_RESERVED = '#'
URL_OTHER_RESERVED = ':@&=/'
URL_RESERVED = URL_PATH_RESERVED + URL_QUERY_RESERVED + URL_OTHER_RESERVED
URL_ESCAPE = '%'
URL_ALNUM = string.letters + string.digits

URL_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_RESERVED + URL_ESCAPE
URL_PATH_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_OTHER_RESERVED + URL_ESCAPE
URL_QUERY_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_OTHER_RESERVED + URL_PATH_RESERVED + URL_ESCAPE
URL_FRAGMENT_VALID_CHARS = URL_ALNUM + URL_SAFE + URL_EXTRA + URL_RESERVED + URL_ESCAPE


# 0-65535
# See: http://www.regular-expressions.info/numericranges.html
PORT_RE = "%s" % "|".join([
    "6553[0-5]",
    "655[0-2][0^9]",
    "65[0-4][0-9][0-9]",
    "6[0-4][0-9][0-9][0-9]",
    "[1-5][0-9][0-9][0-9][0-9]",
    "[1-9][0-9][0-9][0-9]",
    "[1-9][0-9][0-9]",
    "[1-9][0-9]",
    "[1-9]",
])

# See: http://www.shauninman.com/archive/2006/05/08/validating_domain_names
# See: http://www.iana.org/domains/root/db/
DOMAIN_RE = '(?:[a-z0-9](?:[-a-z0-9]*[a-z0-9])?\\.)+(?:(?:aero|arpa|a[cdefgilmnoqrstuwxz])|(?:cat|com|coop|b[abdefghijmnorstvwyz]|biz)|(?:c[acdfghiklmnorsuvxyz])|d[ejkmoz]|(?:edu|e[ceghrstu])|f[ijkmor]|(?:gov|g[abdefghilmnpqrstuwy])|h[kmnrtu]|(?:info|int|i[delmnoqrst])|(?:jobs|j[emop])|k[eghimnprwyz]|l[abcikrstuvy]|(?:mil|mobi|museum|m[acdghklmnopqrstuvwxyz])|(?:name|net|n[acefgilopruz])|(?:om|org)|(?:pro|p[aefghklmnrstwy])|qa|r[eouw]|s[abcdeghijklmnortvyz]|(?:travel|t[cdfghjklmnoprtvwz])|u[agkmsyz]|v[aceginu]|w[fs]|y[etu]|z[amw])'
# Domain with port number
DOMAIN_PORT_RE = '(%s)(?::(%s))?' % (DOMAIN_RE, PORT_RE)

# See: http://www.regular-expressions.info/regexbuddy/ipaccurate.html
IP_ADDRESS_RE = '(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)'
# IP address with port number
IP_PORT_RE = '(%s)(?::(%s))?' % (IP_ADDRESS_RE, PORT_RE)

# Domain or IP address
IP_DOMAIN_RE = '(%s)|(%s)' % (DOMAIN_RE, IP_ADDRESS_RE)

# Domain or IP address with port number
URL_DOMAIN_RE = '(?:%s)(?::(%s))?' % (IP_DOMAIN_RE, PORT_RE)
URL_RE = r'(%s)\:\/\/(%s)(/[%s]*)?(?:\?([%s]*))?(?:\#([%s]*))?' % (
    HTTP_SCHEME_RE,
    URL_DOMAIN_RE,
    re.escape(URL_PATH_VALID_CHARS),
    re.escape(URL_QUERY_VALID_CHARS),
    re.escape(URL_FRAGMENT_VALID_CHARS),
)
URL_RE_CMP = re.compile(URL_RE)

LOOSE_DOMAIN_RE = r"(?:localhost|[\w-]+\.[\w-]+(?:\.[\w-]+)*)"
LOOSE_PORT_RE = "\d+"
LOOSE_URL_RE = r'(%s)\:\/\/(%s)/([%s]*)(?:\?([%s]*))?(?:\#([%s]*))?' % (
    HTTP_SCHEME_RE,
    '(?:(%s)|(%s))(?::(%s))?' % (LOOSE_DOMAIN_RE, IP_ADDRESS_RE, LOOSE_PORT_RE),
    re.escape(URL_PATH_VALID_CHARS),
    re.escape(URL_QUERY_VALID_CHARS),
    re.escape(URL_FRAGMENT_VALID_CHARS),
)
LOOSE_URL_RE_CMP = re.compile(LOOSE_URL_RE)


def sanitize_html(htmlSource, encoding=None, type="text/html",
                  valid_tags=DEFAULT_VALID_TAGS, valid_styles=DEFAULT_VALID_STYLES,
                  add_nofollow=False):
    """
    Clean bad html content. Currently this simply strips tags that
    are not in the VALID_TAGS setting.

    This function is used as a replacement for feedparser's _sanitizeHTML
    and fixes problems like unclosed tags and gives finer grained control
    over what attributes can appear in what tags.

    Returns the sanitized html content.

    encoding is the encoding of the htmlSource

    type is the mimetype of the content. It is ignored and is present only
    for compatibility with feedparser.

    valid_tags is a dictionary containing keys of valid tag names which
    have a value that is a list of valid attribute names. An empty list
    indicates that no attributes are allowed. A value of None indicates
    that all attributes are allowed.

    valid_styles is a list of valid css styles that can be included in
    style tags. Style names that are not included in this list are
    stripped from the html content.

    add_nofollow can be provided which can either be a boolean value
    or a regex that can be matched against a url. If the regex matches
    then nofollow is added to the url.
    """
    from BeautifulSoup import BeautifulSoup, Comment

    js_regex = re.compile(r'[\s]*(&#x.{1,7})?'.join(list('javascript')))
    # Sanitize html with BeautifulSoup
    if encoding:
        soup = BeautifulSoup(htmlSource, fromEncoding=encoding)
    else:
        soup = BeautifulSoup(htmlSource)

    def entities(text):
        return text.replace('<', '&lt;')\
                   .replace('>', '&gt;')\
                   .replace('"', '&quot;')\
                   .replace("'", '&apos;')

    # Sanitize html text by changing bad text to entities.
    # BeautifulSoup will do this for href and src attributes
    # on anchors and image tags but not for text.
    for text in soup.findAll(text=True):
        text.replaceWith(entities(text))

    # コメントを削る
    for comment in soup.findAll(text=lambda text: isinstance(text, Comment)):
        comment.extract()

    for tag in soup.findAll(True):
        if tag.name not in valid_tags:
            tag.hidden = True
        else:
            tag.attrs = [(attr, js_regex.sub('', val))
                         for attr, val in tag.attrs
                         if attr in valid_tags[tag.name]]

    # Add rel="nofollow" links
    if add_nofollow:
        findall_kwargs = {}
        if not isinstance(add_nofollow, bool):
            findall_kwargs["attrs"] = {"href": add_nofollow}
        for tag in soup.findAll("a", **findall_kwargs):
            rel = tag.get('rel', '').split()
            if 'nofollow' not in rel:
                rel.append('nofollow')
                tag['rel'] = ' '.join(rel)

    # Clean up CSS style tags
    for tag in soup.findAll(attrs={"style": re.compile(".*")}):

        old_styles = [s.strip() for s in tag["style"].split(";")]

        # style_order preserves uniqueness
        styles = {}
        # style_order preserves order
        style_order = []
        for style in old_styles:
            if ':' in style:
                style_name, style_value = style.split(':', 1)
                # Style names are case insensitive so
                # change to lowercase
                style_name = style_name.strip().lower()
                if style_name in valid_styles:
                    style_value = style_value.strip()
                    if style_name in styles:
                        styles[style_name] = style_value
                        style_order.remove(style_name)
                        style_order.append(style_name)
                    else:
                        styles[style_name] = style_value
                        style_order.append(style_name)

        if styles:
            tag["style"] = ';'.join([('%s:%s' % (s, styles[s])) for s in style_order]) + ';'
        else:
            del tag["style"]

    # Sanitize html text by changing bad text to entities.
    # BeautifulSoup will do this for href and src attributes
    # on anchors and image tags but not for text.
    for text in soup.findAll(text=True):
        text.replaceWith(escape_entities(text))

    return soup.renderContents().decode('utf8')

URLIZE_TMPL = '<a href="%(link_url)s"%(attrs)s>%(link_text)s</a>'


def urlize(text, trim_url_limit=None, attrs={}, url_re=URL_RE_CMP, autoescape=False):
    """text内URLを抽出してアンカータグで囲む

    URLのデリミタは半角カンマ、<>(エスケープ済み含む)、\s、全角スペース、行末で、これらが末尾にマッチしない場合はURLとして認識しません。
    URL部分は.+の最小マッチ、もしくはtrim_url_limitが指定された場合は{,trim_url_limit}の最小マッチとなります。

    -args

        text:           urlize対象文字列
        trim_url_limit: urlとして認識する文字数に上限を設ける場合は数値をセット
        nofollow:       Trueを与えるとタグにrel="nofollow"を付加
        autoescape:     Trueを与えるとタグエスケープを行います。

    """
    from strutils import abbrev

    if autoescape:
        text = escape(text)

    def _repl(m):
        return URLIZE_TMPL % {
            "link_url": m.group(),
            "attrs": "".join(map(lambda x: ' %s="%s"' % x, attrs.iteritems())),
            "link_text": (abbrev(m.group(), trim_url_limit)
                          if trim_url_limit is not None else m.group()),
        }

    return url_re.sub(_repl, text)



# vim:fileencoding=utf-8

"""
String utilities.
"""

import string

from random import sample

__all__ = (
    'StrAndUnicode',
    'force_unicode',
    'smart_str',
    'trim',
    'force_int',
    'make_random_key',
    'abbrev',
)

try:
    from django.utils.encoding import StrAndUnicode, force_unicode, smart_str
except ImportError:
    import types
    import datetime
    from decimal import Decimal

    class StrAndUnicode(object):
        """
        A class whose __str__ returns its __unicode__ as a UTF-8 bytestring.

        Useful as a mix-in.
        """
        def __str__(self):
            return self.__unicode__().encode('utf-8')

    def is_protected_type(obj):
        """Determine if the object instance is of a protected type.

        Objects of protected types are preserved as-is when passed to
        force_unicode(strings_only=True).
        """
        return isinstance(obj, (
            types.NoneType,
            int, long,
            datetime.datetime, datetime.date, datetime.time,
            float, Decimal)
        )

    def force_unicode(s, encoding='utf-8', strings_only=False, errors='strict'):
        """
        Similar to smart_unicode, except that lazy instances are resolved to
        strings, rather than kept as lazy objects.

        If strings_only is True, don't convert (some) non-string-like objects.
        """
        if strings_only and is_protected_type(s):
            return s
        if not isinstance(s, basestring,):
            if hasattr(s, '__unicode__'):
                s = s.__unicode__()
            else:
                try:
                    s = unicode(str(s), encoding, errors)
                except UnicodeEncodeError:
                    if not isinstance(s, Exception):
                        raise
                    # If we get to here, the caller has passed in an Exception
                    # subclass populated with non-ASCII data without special
                    # handling to display as a string. We need to handle this
                    # without raising a further exception. We do an
                    # approximation to what the Exception's standard str()
                    # output should be.
                    s = ' '.join([force_unicode(arg, encoding, strings_only, errors) for arg in s])
        elif not isinstance(s, unicode):
            # Note: We use .decode() here, instead of unicode(s, encoding,
            # errors), so that if s is a SafeString, it ends up being a
            # SafeUnicode at the end.
            s = s.decode(encoding, errors)
        return s

    def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
        """
        Returns a bytestring version of 's', encoded as specified in 'encoding'.

        If strings_only is True, don't convert (some) non-string-like objects.
        """
        if strings_only and isinstance(s, (types.NoneType, int)):
            return s
        elif not isinstance(s, basestring):
            try:
                return str(s)
            except UnicodeEncodeError:
                if isinstance(s, Exception):
                    # An Exception subclass containing non-ASCII data that doesn't
                    # know how to print itself properly. We shouldn't raise a
                    # further exception.
                    return ' '.join([smart_str(arg, encoding, strings_only, errors) for arg in s])
                return unicode(s).encode(encoding, errors)
        elif isinstance(s, unicode):
            return s.encode(encoding, errors)
        elif s and encoding != 'utf-8':
            return s.decode('utf-8', errors).encode(encoding, errors)
        else:
            return s


def trim(s, encoding="utf-8"):
    """
    全角・半角も含めてトリミング
    """
    return force_unicode(s, encoding).strip()


def force_int(num, default=None):
    """
    Force a value to an integer. If the value
    cannot be converted return the default value.
    """
    try:
        return int(num)
    except (ValueError, TypeError):
        return default


def make_random_key(size=128, values=None):
    """
    Creates a random string value.

    This does not create secure token values. If you need to
    create a secure key you should use the hmac module
    to create a salted HMAC-SHA1 token.
    """
    if values is None:
        values = string.letters + string.digits
    keys = ""
    src = list(values)
    while True:
        diff = size - len(keys)
        if diff <= 0:
            break
        keys += "".join(sample(src, (diff < 20 and diff or 20)))
    return keys


def abbrev(s, num=255, end="..."):
    """
    文章を要約する
    返す文字列の長さは、num以上にならないのを保証します。

    >>> abbrev('spamspamspam', 6)
    'spa...'
    >>> abbrev('spamspamspam', 12)
    'spamspamspam'
    >>> abbrev('eggseggseggs', 1)
    'e'
    >>> abbrev('eggseggseggs', 2)
    'eg'
    >>> abbrev('eggseggseggs', 3)
    'egg'
    >>> abbrev('eggseggseggs', 4)
    'e...'
    >>> abbrev('eggseggseggs', 2, '.')
    'e.'
    """
    index = num - len(end)
    if len(s) > num:
        s = (s[:index] + end) if index > 0 else s[:num]
    return s



#:coding=utf8:

from unittest import TestCase

from decimal import Decimal

from beproud.utils.decimalutils import *

class TestDecimalUtils(TestCase):

    def assertDecimalValue(self, d, val):
        self.assertTrue(isinstance(d, Decimal))
        self.assertEqual(d, Decimal(val))
  
    def test_force_decimal_string(self):
        for val in ["1.04949", "9", "-1.04"]:
            self.assertDecimalValue(force_decimal(val), val)
        
        for val in ["val", u"日本語"]:
            try:
                dec = force_decimal(val)
            except ValueError:
                pass
            else:
                self.fail("Expected failure for %s" % val)

    def test_numbers_only(self):
        for val in ["1.04949", "9", "-1.04"]:
            self.assertDecimalValue(force_decimal(val, numbers_only=True), val)
        
        for val in ["val", u"日本語"]:
            self.assertEqual(force_decimal(val, numbers_only=True), val)



#:coding=utf8:

from datetime import datetime
from unittest import TestCase

from beproud.utils.datastructures import *

class CachedPropObj(object):
    id = 0

    def spam(self):
        return datetime.now()

    @cached_property
    def cached_spam(self):
        return datetime.now()

    def eggs(self):
        self.__class__.id += 1
        return self.__class__.id

    @cached_property
    def cached_eggs(self):
        self.__class__.id += 1
        return self.__class__.id

class CachedPropertyTest(TestCase):

    def test_cached_property_get(self):
        obj = CachedPropObj()
        x = obj.cached_spam
        self.assertTrue(x)
        self.assertTrue(x is obj.cached_spam)

        y = obj.cached_eggs
        self.assertTrue(y)
        self.assertTrue(y is obj.cached_eggs)

    def test_cached_property_set(self):
        obj = CachedPropObj()
        obj.cached_spam = "spam"
        self.assertEqual(obj.cached_spam, "spam")

        self.assertTrue(obj.cached_eggs != "eggs")
        obj.cached_eggs = "eggs"
        self.assertEqual(obj.cached_eggs, "eggs")



#!/usr/bin/env python
#:coding=utf-8:

import unittest
import doctest

def additional_tests():
    import beproud.utils.decimalutils

    suite = unittest.TestSuite()
    suite.addTest(doctest.DocTestSuite(beproud.utils.decimalutils))
    return suite

def main():
    suite = unittest.TestSuite()
    runner = unittest.TextTestRunner()
    runner.run(suite)

if __name__ == '__main__':
    import os
    import sys
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
    main()



#:coding=utf8:

# vim:fileencoding=utf-8
from unittest import TestCase

from beproud.utils.word import * 

TEST_FEATURE_MAP = (
    "word_class",
    "sub_word_class1",
    "sub_word_class2",
    "sub_word_class3",
    "conjugated",
    "conjugation",
    "base",
    "reading",
    "pronunciation",
)

class TokenizerTestCase(TestCase):
    
    def test_encoding(self):
        tokenizer = Tokenizer(encoding="cp932")
        tokens = tokenizer.tokenize(u"これはテスト文章です".encode("cp932"))
        self.assertEqual(len(list(tokens)), 5)

    def test_features(self):
        tokenizer = Tokenizer()
        tokens = tokenizer.tokenize(u"これはテスト文章です")
        for token in tokens:
            for feature in TEST_FEATURE_MAP:
                self.assertTrue(feature in token.features)

    def test_word_classes(self):
        tokenizer = Tokenizer(word_classes=["名詞"])
        tokens = tokenizer.tokenize(u"これはテスト文章です")
        token_list = list(tokens)
        self.assertEquals(len(token_list), 3)
        self.assertEquals(token_list[0], u"これ")
        self.assertEquals(token_list[1], u"テスト")
        self.assertEquals(token_list[2], u"文章")

    def test_min_word_length(self):
        tokenizer = Tokenizer(min_word_length=3)
        tokens = tokenizer.tokenize(u"これはテスト文章です")
        token_list = list(tokens)
        self.assertEquals(len(token_list), 1)
        self.assertEquals(token_list[0], u"テスト")

    def test_callback(self):
        def cb(word, features):
            self.assertTrue(isinstance(features, dict))
            self.assertTrue(isinstance(word, str))
            utf = unicode(word, "utf8") # unicode test
            cb.count += 1
            return Token.new(word, "utf8", features=features)
        cb.count = 0

        tokenizer = Tokenizer(token_callback=cb)
        tokens = tokenizer.tokenize(u"これはテスト文章です")
        self.assertEquals(len(list(tokens)), 5)
        self.assertEquals(cb.count, 5)
    
    def test_callback_none(self):
        def cb(word, features):
            pass
        tokenizer = Tokenizer(token_callback=cb)
        tokens = tokenizer.tokenize(u"これはテスト文章です")
        self.assertEquals(len(list(tokens)), 0)



#:coding=utf8:

"""
Utilities for decimal values. Many values have been copied from
the python documentation.

https://docs.python.org/2/library/decimal.html#recipes


Many of these functions are *very* slow compared to the
equivalent operations on float values. Use them sparingly.
"""

from decimal import (
    Decimal,
    Context,
    getcontext,
    setcontext,
    InvalidOperation,
    Inexact,
)

__all__ = (
    'force_decimal',
    'pi',
    'exp',
    'cos',
    'sin',
    'log',
    'ln',
)


def force_decimal(d, precision_loss=True, numbers_only=False):
    """
    Coerces a value to a Decimal object.

    If precision_loss is True then float objects are first converted
    to strings before being converted to Decimal objects. This can cause
    a loss of precision as the float objects are rounded. The 'str'
    method is used since it provides better precision than "%f" % float.

    If you wish to preserve as much information as possible you can call
    the method with precision_loss=False and method will be used that
    can convert the float without loosing information. This can, however,
    add extra information due to issues with binary floating point.

    See: Is there a way to convert a regular float to a Decimal?
    http://docs.python.org/library/decimal.html#decimal-faq

    If numbers_only is True, don't convert (some) non-number-like objects.
    Strings that can be converted to Decimals are still converted.
    """
    try:
        if isinstance(d, Decimal):
            return d
        elif isinstance(d, (int, long)):
            return Decimal(d)
        elif isinstance(d, float):
            return Decimal(str(d)) if precision_loss else float_to_decimal(d)
        elif isinstance(d, (basestring, unicode)):
            return Decimal(d)
    except InvalidOperation:
        pass

    if numbers_only:
        return d
    else:
        raise ValueError("Cannot convert object to Decimal: %r" % d)


def float_to_decimal(f):
    """
    Convert a floating point number to a Decimal with no loss of information
    """
    n, d = f.as_integer_ratio()
    numerator, denominator = Decimal(n), Decimal(d)
    ctx = Context(prec=60)
    result = ctx.divide(numerator, denominator)
    while ctx.flags[Inexact]:
        ctx.flags[Inexact] = False
        ctx.prec *= 2
        result = ctx.divide(numerator, denominator)
    return +result


def pi():
    """
    Compute Pi to the current precision.
    Taken from: http://docs.python.org/library/decimal.html#decimal-recipes

    >>> print pi()
    3.141592653589793238462643383
    """
    getcontext().prec += 2  # extra digits for intermediate steps
    three = Decimal(3)      # substitute "three=3.0" for regular floats
    lasts, t, s, n, na, d, da = 0, three, 3, 1, 0, 0, 24
    while s != lasts:
        lasts = s
        n, na = n+na, na+8
        d, da = d+da, da+32
        t = (t * n) / d
        s += t
    getcontext().prec -= 2
    return +s               # unary plus applies the new precision


def exp(x):
    """
    Return e raised to the power of x.  Result type matches input type.
    Taken from: http://docs.python.org/library/decimal.html#decimal-recipes

    >>> print exp(Decimal(1))
    2.718281828459045235360287471
    >>> print exp(Decimal(2))
    7.389056098930650227230427461
    >>> print exp(2.0)
    7.38905609893
    >>> print exp(2+0j)
    (7.38905609893+0j)

    """
    getcontext().prec += 2
    i, lasts, s, fact, num = 0, 0, 1, 1, 1
    while s != lasts:
        lasts = s
        i += 1
        fact *= i
        num *= x
        s += num / fact
    getcontext().prec -= 2
    return +s


def cos(x):
    """
    Return the cosine of x as measured in radians.
    Taken from: http://docs.python.org/library/decimal.html#decimal-recipes

    >>> print cos(Decimal('0.5'))
    0.8775825618903727161162815826
    >>> print cos(0.5)
    0.87758256189
    >>> print cos(0.5+0j)
    (0.87758256189+0j)
    """
    getcontext().prec += 2
    i, lasts, s, fact, num, sign = 0, 0, 1, 1, 1, 1
    while s != lasts:
        lasts = s
        i += 2
        fact *= i * (i-1)
        num *= x * x
        sign *= -1
        s += num / fact * sign
    getcontext().prec -= 2
    return +s


def sin(x):
    """
    Return the sine of x as measured in radians.
    Taken from: http://docs.python.org/library/decimal.html#decimal-recipes

    >>> print sin(Decimal('0.5'))
    0.4794255386042030002732879352
    >>> print sin(0.5)
    0.479425538604
    >>> print sin(0.5+0j)
    (0.479425538604+0j)
    """
    getcontext().prec += 2
    i, lasts, s, fact, num, sign = 1, 0, x, 1, x, 1
    while s != lasts:
        lasts = s
        i += 2
        fact *= i * (i-1)
        num *= x * x
        sign *= -1
        s += num / fact * sign
    getcontext().prec -= 2
    return +s


def log(self, base=10, context=None):
    """
    Returns a log of arbitrary precision using the method
    described here: http://www.programmish.com/?p=25

    Input must be a decimal

    >>> print log(Decimal("1.204"))
    0.0806264869218057475447822012
    >>> print log(Decimal("1.204"), 2)
    0.2678353920976150027151526692

    >>> import math
    >>> abs(math.log(1.204, 10) - float(log(Decimal("1.204")))) < 1e-15
    True
    >>> abs(math.log(1.827528759292, 10) - float(log(Decimal("1.827528759292")))) < 1e-15
    True
    """
    old_context = None
    if context is not None:
        old_context = getcontext()
        setcontext(context)

    cur_prec = getcontext().prec
    getcontext().prec += 2
    try:
        retValue = self
        baseDec = force_decimal(base)

        integer_part = Decimal(0)
        while retValue < 1:
            integer_part = integer_part - 1
            retValue = retValue * baseDec
        while retValue >= baseDec:
            integer_part = integer_part + 1
            retValue = retValue / baseDec

        retValue = retValue ** 10
        decimal_frac = Decimal(0)
        partial_part = Decimal(1)
        while cur_prec > 0:
            partial_part = partial_part / Decimal(10)
            digit = Decimal(0)
            while retValue >= baseDec:
                digit += 1
                retValue = retValue / baseDec
            decimal_frac = decimal_frac + digit * partial_part
            retValue = retValue ** 10
            cur_prec -= 1
    finally:
        # restore contexts
        getcontext().prec -= 2
        if old_context is not None:
            setcontext(old_context)

    return integer_part + decimal_frac


def ln(self, context=None):
    """
    Returns the natural log of the given Decimal

    Input must be a decimal

    >>> print ln(Decimal("1.204"))
    0.1856493468866292953586851357

    >>> import math
    >>> abs(math.log(1.204) - float(ln(Decimal("1.204")))) < 1e-15
    True
    >>> abs(math.log(1.827528759292) - float(ln(Decimal("1.827528759292")))) < 1e-15
    True
    """
    if hasattr(self, "ln"):
        return self.ln(context)
    else:
        return log(self, base=exp(Decimal("1.0")))



#:coding=utf8:

from unittest import TestCase

from decimal import Decimal

from beproud.utils.dbutils.mysql import *

class TestMySQLCursor(TestCase):
    u"""
    DB: dbutils_test

    MySQL データベースは必須
    CREATE DATABASE bputils_test CHARACTER SET utf8;
    CREATE TABLE person (name CHAR(20) CHARACTER SET utf8 COLLATE utf8_bin);
    INSERT INTO person (name) VALUES ('ian');
    INSERT INTO person (name) VALUES ('tokibito');
    INSERT INTO person (name) VALUES ('aodag');
    """
    def setUp(self):
        self.conn = MySQLConnection({"host":"localhost", "user":"root", "db":"bputils_test", "charset":"utf8"}) 

    def test_iteration(self):
        cursor = MySQLCursor(self.conn)
        cursor.query("SELECT * from person")
        for row in cursor:
            self.assertTrue(row["name"])

    def test_iteration2(self):
        cursor = MySQLCursor(self.conn, rows=2)
        cursor.query("SELECT * from person")
        for row in cursor:
            self.assertTrue(row["name"])

    def test_restart(self):
        cursor = MySQLCursor(self.conn)
        cursor.query("SELECT * from person")

        self.assertTrue(cursor.next()["name"])
        self.assertTrue(cursor.next()["name"])
        self.assertTrue(cursor.next()["name"])

        cursor.query("SELECT * from person")
        self.assertTrue(cursor.next()["name"])
        self.assertTrue(cursor.next()["name"])
        self.assertTrue(cursor.next()["name"])




#:coding=utf-8:

TUPLE=0
DICT=1
DICT_WITH_TABLE=2

def MySQLConnection(connect_info):
    """ 単純にMySQLのコネクションを作って返します。 """
    import MySQLdb
    return MySQLdb.connect(**connect_info)

class MySQLCursor(object):
    """
    MySQLdb用のカーソルクラス。

    connection = MySQLConnection({
        'db' : "imagawa",
        'host' : "127.0.0.1",
        'port' : 3306,
        'user' : "user",
        'passwd' : "password"
    })
    cursor = MySQLCursor(connection)
    cursor.query('select id, name, email from huge_data')
    for i in cursor:
        # do something...
    """

    _next = None
    _index = 0
    def __init__(self, connection, rows=1, value_type=DICT):
        self.connection = connection
        self.rows = rows
        self.value_type = value_type
        self.rs = None
        self.row = None
    
    def query(self, sql):
        self.connection.query(sql)
        self.rs = self.connection.use_result()
        self._next = None
    
    def __iter__(self):
        return self

    def has_next(self):
        if self._next is None:
            if not self.row or self._index+1 > len(self.row):
                self.row = list(self.rs.fetch_row(self.rows, self.value_type) or [])
                self._index = 0
            self._next = bool(self.row)
        return self._next

    def next(self):
        if self.has_next():
            self._next = None
            return_val = self.row[self._index]
            self._index += 1
            self.has_next() # Check for next values
            return return_val
        else:
            raise StopIteration  



# vim:fileencoding=utf-8
from datetime import datetime,date,time
from unittest import TestCase

from beproud.utils.javascript import (
    json,
    escapejs_json,
    force_js,
    SafeJSONEncoder,
)

class JavascriptUtilsTestCase(TestCase):

    def test_escapejs_json(self):
        self.assertEqual(escapejs_json(u"<テスト>"), u'\\u003cテスト\\u003e')
        self.assertEqual(escapejs_json(u"Q&A"), u"Q\\u0026A")
        self.assertEqual(escapejs_json(u"データ"), u"データ")
        self.assertEqual(escapejs_json("<test>"), u"\\u003ctest\\u003e")
        self.assertEqual(escapejs_json(u"テスト & エスケープ"), u"テスト \\u0026 エスケープ")

    def test_force_js(self):
        self.assertEqual(force_js({"msg": u"メッセージ"}), '{"msg": "\\u30e1\\u30c3\\u30bb\\u30fc\\u30b8"}')
        self.assertEqual(force_js(datetime(2009, 12, 22, 16, 35, 02)), '"2009-12-22 16:35:02"') 
        self.assertEqual(force_js(date(2009, 12, 22)), '"2009-12-22"') 
        self.assertEqual(force_js(time(16, 35, 02)), '"16:35:02"')

        self.assertEqual(force_js(u"<title>"), u'"\\u003ctitle\\u003e"')

    def test_datetime(self):
        from datetime import datetime,date,time
        self.assertEqual(force_js({"msg": u"メッセージ"}), '{"msg": "\\u30e1\\u30c3\\u30bb\\u30fc\\u30b8"}')
        self.assertEqual(force_js(datetime(2009, 12, 22, 16, 35, 02)), '"2009-12-22 16:35:02"') 
        self.assertEqual(force_js(date(2009, 12, 22)), '"2009-12-22"') 
        self.assertEqual(force_js(time(16, 35, 02)), '"16:35:02"')

    def test_decimal(self):
        from decimal import Decimal
        self.assertEqual(force_js(Decimal("5")), '5')
        self.assertEqual(force_js({"dec": Decimal("1.5")}), '{"dec": 1.5}')

    def test_force_js_type(self):
        self.assertEqual(force_js("", "bool"), "false")
        self.assertEqual(force_js("5", "int"), '5')
        self.assertEqual(force_js("test", "array"), '["t", "e", "s", "t"]')
        self.assertEqual(force_js(5, "string"), '"5"')

    def test_safe_json_encoder(self):
        data = {
            "title": "<title>",
            "datetime": datetime(2009, 2, 19, 10, 22),
        }
        self.assertEqual(
            json.dumps(data, cls=SafeJSONEncoder),
            u'{"datetime": "2009-02-19 10:22:00", "title": "\\u003ctitle\\u003e"}',
        )



# vim:fileencoding=utf-8
from unittest import TestCase

from beproud.utils.strutils import *

class StringUtilsTestCase(TestCase):
    
    def test_trim(self):
        self.assertEqual(trim(u" テスト "), u"テスト")
        self.assertEqual(trim(u"テスト "), u"テスト")
        self.assertEqual(trim(u" テスト"), u"テスト")
        self.assertEqual(trim(u"　テスト　"), u"テスト")
        self.assertEqual(trim(u"テスト　"), u"テスト")
        self.assertEqual(trim(u"　テスト"), u"テスト")
        self.assertEqual(trim(u"　テスト "), u"テスト")

    def test_trim_none(self):
        self.assertEqual(trim(None), 'None')

    def test_force_int(self):
        self.assertEqual(force_int(192), 192)
        self.assertEqual(force_int("192"), 192)
        self.assertEqual(force_int("abc"), None)
        self.assertEqual(force_int("abc", "def"), "def")

    def test_force_int_none(self):
        self.assertEqual(force_int(None, 1), 1)

    def test_force_unicode(self):
        class UnicodeTest(StrAndUnicode):
            def __unicode__(self):
                return u"Unicodeテスト"
        self.assertEqual(force_unicode(UnicodeTest()), u"Unicodeテスト")
        self.assertEqual(force_unicode(22), u"22")
        self.assertEqual(force_unicode(22, strings_only=True), 22)

        self.assertEqual(force_unicode(u"テスト".encode("cp932"), encoding="cp932"), u"テスト") 
    
    def test_make_random_key(self):
        self.assertEqual(len(make_random_key(8)), 8)
        self.assertEqual(len(make_random_key(291)), 291)
        self.assertEqual(len(make_random_key(1)), 1)
    
    def test_make_random_key_empty(self):
        self.assertEqual(make_random_key(0), "")
        self.assertEqual(make_random_key(-1), "")

    def test_abbrev(self):
        self.assertEqual(abbrev('spamspamspam', 6), 'spa...')
        self.assertEqual(abbrev('spamspamspam', 12), 'spamspamspam')
        self.assertEqual(abbrev('eggseggseggs', 11, '....'), 'eggsegg....')
        self.assertEqual(abbrev('eggseggseggs', 1), 'e')
        self.assertEqual(abbrev('eggseggseggs', 2, '.'), 'e.')

    def test_abbrev_unicode(self):
        self.assertEqual(abbrev(u'テストテストテスト', 6), u'テスト...')
        self.assertEqual(abbrev(u'テストテストテスト', 9), u'テストテストテスト')
        self.assertEqual(abbrev(u'テストテストテスト', 8, '....'), u'テストテ....')
        self.assertEqual(abbrev(u'テストテストテスト', 1), u'テ')
        self.assertEqual(abbrev(u'テストテストテスト', 2, '.'), u'テ.')



# vim:fileencoding=utf-8
from functools import update_wrapper

def cached_property(f, name=None):
    """
    A decorator that caches the output of a method to the __dict__
    of the class instance.
    """
    if name is None:
        name = f.__name__ 
    def _get(self):
        try:
            return self.__dict__[name]
        except KeyError:
            value = f(self)
            self.__dict__[name] = value 
            return value
    update_wrapper(_get, f)
    def _set(self, value):
        self.__dict__[name] = value
    return property(_get, _set)

class DotDict(dict):
    def __getattr__(self, attr):
        return self.get(attr, None)
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__



#:coding=utf8:

import re
from unittest import TestCase

from beproud.utils.html import (
    LOOSE_DOMAIN_RE,
    DOMAIN_RE,
    IP_ADDRESS_RE,
    URL_RE,
    DEFAULT_VALID_TAGS,
    DEFAULT_VALID_STYLES,
    urlize,
    sanitize_html,
)

class UrlReTest(TestCase):
    def test_loose_domain_re(self):
        domain_re = re.compile(LOOSE_DOMAIN_RE)
        self.assertEqual(domain_re.match("localhost").group(), 'localhost')
        self.assertTrue(domain_re.match("blah") is None)
        self.assertEqual(domain_re.match("beproud.jp").group(), 'beproud.jp')
        self.assertEqual(domain_re.match("dev-xxx.beproud.jp").group(), 'dev-xxx.beproud.jp')
        self.assertEqual(domain_re.match("dev_xxx.beproud.jp").group(), 'dev_xxx.beproud.jp')
        self.assertEqual(domain_re.match("dev_xxx.be_proud.j_p").group(), 'dev_xxx.be_proud.j_p')
        self.assertEqual(domain_re.match("www2.beproud.jp").group(), 'www2.beproud.jp')
        self.assertEqual(domain_re.match("www2.static.beproud.jp").group(), 'www2.static.beproud.jp')
        self.assertEqual(domain_re.match("www2.static.beproud.zcode").group(), 'www2.static.beproud.zcode')

    def test_domain_re(self):
        domain_re = re.compile(DOMAIN_RE)
        self.assertTrue(domain_re.match("localhost") is None)
        self.assertTrue(domain_re.match("blah") is None)
        self.assertEqual(domain_re.match("zcode.jp").group(), 'zcode.jp')
        self.assertEqual(domain_re.match("www2.zcode.jp").group(), 'www2.zcode.jp')
        self.assertEqual(domain_re.match("www2.static.zcode.jp").group(), 'www2.static.zcode.jp')
        self.assertEqual(domain_re.match("dev-xxx.beproud.jp").group(), 'dev-xxx.beproud.jp')
        self.assertTrue(domain_re.match("dev_xxx.beproud.jp") is None)
        self.assertEqual(domain_re.match("devxxx.bep_roud.jp").group(), 'devxxx.be')
        self.assertEqual(domain_re.match("devxxx.beproud.j_p").group(), 'devxxx.be')
        self.assertTrue(domain_re.match("www2.zcode") is None)

    def test_ip_address_re(self):
        ip_address_re = re.compile(IP_ADDRESS_RE)
        self.assertTrue(ip_address_re.match("localhost") is None)
        self.assertTrue(ip_address_re.match("blah") is None)
        self.assertTrue(ip_address_re.match("123.beproud.jp") is None)
        self.assertTrue(ip_address_re.match("123.148.224.249") is not None)
        self.assertTrue(ip_address_re.match("255.255.255.255") is not None)
        self.assertTrue(ip_address_re.match("123.148.294.226") is None)
        self.assertTrue(ip_address_re.match("123.999.224.249") is None)

    def test_url_re(self):
        url_re = re.compile(URL_RE)
        self.assertEquals(
            url_re.match("http://www.beproud.jp/company/?param=test#fragment").groups(),
            ('http', 'www.beproud.jp', 'www.beproud.jp', None, None, '/company/', 'param=test', 'fragment'),
        )

        self.assertEquals(
            url_re.match("http://www.beproud.jp:8000/company/").groups(),
            ('http', 'www.beproud.jp:8000', 'www.beproud.jp', None, '8000', '/company/', None, None),
        )

        self.assertEquals(
            url_re.match("http://www.beproud.jp:8000/company/?quer*?y=str#frag?m#ent").groups(),
            ('http', 'www.beproud.jp:8000', 'www.beproud.jp', None, '8000', '/company/', 'quer*?y=str', 'frag?m#ent'),
        )

        self.assertEquals(
            url_re.match("http://123.123.123.123:8000/company/?quer*?y=str#frag?m#ent").groups(),
            ('http', '123.123.123.123:8000', None, '123.123.123.123', '8000', '/company/', 'quer*?y=str', 'frag?m#ent'),
        )

        self.assertTrue(url_re.match("http://www.テスト.jp:8000/company/") is None)

class UrlizeTest(TestCase):

    def test_urlize(self):
        self.assertEqual(
            urlize(u'ペキペキ12月はカップル割り15％OFFです http://twitter.com/#!/PEKI_SHIBUYA/status/142081523179462656'),
            u'ペキペキ12月はカップル割り15％OFFです <a href="http://twitter.com/#!/PEKI_SHIBUYA/status/142081523179462656">http://twitter.com/#!/PEKI_SHIBUYA/status/142081523179462656</a>',
        )

    def test_domain_check(self):
        self.assertEqual(
            urlize(u'このURL、http://beproud.jpビープラウドホームページ'),
            u'このURL、<a href="http://beproud.jp">http://beproud.jp</a>ビープラウドホームページ',
        )

    def test_attrs(self):
        self.assertEqual(
            urlize(u'ペキペキ12月はカップル割り15％OFFです http://twitter.com/#!/PEKI_SHIBUYA/status/142081523179462656', attrs={'rel': 'nofollow'}),
            u'ペキペキ12月はカップル割り15％OFFです <a href="http://twitter.com/#!/PEKI_SHIBUYA/status/142081523179462656" rel="nofollow">http://twitter.com/#!/PEKI_SHIBUYA/status/142081523179462656</a>',
        )

    def test_url_trim(self):
        self.assertEqual(
            urlize(u'このURL、http://beproud.jp/?query=strビープラウドホームページ', trim_url_limit=23),
            u'このURL、<a href="http://beproud.jp/?query=str">http://beproud.jp/?q...</a>ビープラウドホームページ',
        )


class HTMLSanitizationTest(TestCase):
    valid_tags = DEFAULT_VALID_TAGS
    valid_styles = DEFAULT_VALID_STYLES
    test_html = () 

    def test_html_sanitization(self):
        for html in self.test_html:
            sanitized_html = sanitize_html(
                html[0],
                valid_tags=self.valid_tags,
                valid_styles=self.valid_styles,
            )
            self.assertEqual(sanitized_html, html[1])

class TagStrippingTest(HTMLSanitizationTest):
    test_html = (
        (
            u'<b>This is a test</b>', 
            u'<b>This is a test</b>',
        ),
        (
            u'<script type="text/javascript">alert("DANGER!!");</script> Will Robinson',
            u'alert(&quot;DANGER!!&quot;); Will Robinson',
        ),
        (
            u'<a href="http://www.ianlewis.org/" rel="me" onclick="alert(\'woah!!\')">This is a test</a>',
            u'<a href="http://www.ianlewis.org/" rel="me">This is a test</a>',
        ),
        (
            u'<CRazY>Crazy Text</crazy>',
            u'Crazy Text',
        )
    )

class EntitiesTest(HTMLSanitizationTest):
    test_html = (
        (
            u'<b>Ian\'s Homepage</b>',
            u'<b>Ian&apos;s Homepage</b>',
        ),
        (
            u'"I CaN HaZ SoMe < TeXt?"',
            u'&quot;I CaN HaZ SoMe &lt; TeXt?&quot;',
        ),
        (
            u'"I CaN HaZ SoMe <TeXt>?"',
            u'&quot;I CaN HaZ SoMe ?&quot;',
        ),
        (
            u'Ice Cream & &quot;Chocolate&quot;',
            u'Ice Cream &amp; &quot;Chocolate&quot;',
        ),
        (
            u"hoge&#39;s & fuga's",
            u"hoge&#39;s &amp; fuga&apos;s",
        )
    )

class CSSSanitizationTest(HTMLSanitizationTest):
    valid_tags = {
        'div': ('style',),
        'span': ('style',),
    }
    valid_styles = (
        "color",
        "font-weight",
        "width",
        "background-image",
    )
    test_html = (
        (
            u'<span style="color:#FFF;position:absolute;">My Homepage</span>', 
            u'<span style="color:#FFF;">My Homepage</span>', 
        ),
        (
            u'<span style="color:#FFF;position:absolute;font-weight:bold;">My Homepage</span>', 
            u'<span style="color:#FFF;font-weight:bold;">My Homepage</span>', 
        ),
        (
            u'<span style="color:#FFF;position:absolute   ">My Homepage</span>', 
            u'<span style="color:#FFF;">My Homepage</span>', 
        ),
        (
            u'<span style="  color:#FFF;  position:absolute;\tfont-weight:bold  ">My Homepage</span>', 
            u'<span style="color:#FFF;font-weight:bold;">My Homepage</span>', 
        ),
        (
            u'<span style="  color:#FFF;  position:absolute;\tfont-weight:bold;  aaaaa">My Homepage</span>', 
            u'<span style="color:#FFF;font-weight:bold;">My Homepage</span>', 
        ),
        (
            u'<span style="  color  : #FFF;  position:absolute;\tfont-weight:bold;  aaaaa">My Homepage</span>', 
            u'<span style="color:#FFF;font-weight:bold;">My Homepage</span>', 
        ),
        (
            u'<span style="border-width:5px;width:6px;border-left-width:10px">My Homepage</span>', 
            u'<span style="width:6px;">My Homepage</span>', 
        ),
        # CSS では、最後に出るスタイルが有効なので
        (
            u'<span style="width:5px;width:6px;">My Homepage</span>', 
            u'<span style="width:6px;">My Homepage</span>', 
        ),
        (
            u'<span style="WIDTH:5px;color:#FFF;">My Homepage</span>', 
            u'<span style="width:5px;color:#FFF;">My Homepage</span>', 
        ),
        (
            u'<span style="position:absolute">My Homepage</span>', 
            u'<span>My Homepage</span>', 
        ),
        (
            u'<span style="background-image:url(http://example.com/example.jpg);">My Homepage</span>', 
            u'<span style="background-image:url(http://example.com/example.jpg);">My Homepage</span>', 
        ),
    )

class ExtraHTMLTest(HTMLSanitizationTest):
    test_html = (
        (
            u'<input name=submit type=submit value=検索><span>Test</span>',
            u'<span>Test</span>',
        ),
    )

class NoFollowTest(TestCase):
    valid_tags = DEFAULT_VALID_TAGS
    valid_styles = DEFAULT_VALID_STYLES
    test_html = () 

    def test_nofollow(self):
        html = u'<a href="http://www.ianlewis.org/">This is a test</a>'
        sanitized_html = sanitize_html(html, valid_tags=self.valid_tags, add_nofollow=True)
        self.assertEqual(sanitized_html,
            u'<a href="http://www.ianlewis.org/" rel="nofollow">This is a test</a>')

        html = u'<a href="http://www.ianlewis.org/" rel="me">This is a test</a>'
        sanitized_html = sanitize_html(html, valid_tags=self.valid_tags, add_nofollow=True)
        self.assertEqual(sanitized_html,
            u'<a href="http://www.ianlewis.org/" rel="me nofollow">This is a test</a>')

    def test_nofollow_existing(self):
        html = u'<a href="http://www.ianlewis.org/" rel="me nofollow">This is a test</a>'
        sanitized_html = sanitize_html(html, valid_tags=self.valid_tags, add_nofollow=True)
        self.assertEqual(sanitized_html,
            u'<a href="http://www.ianlewis.org/" rel="me nofollow">This is a test</a>')

class SanitizeHtmlEncodingTest(TestCase):
    valid_tags = DEFAULT_VALID_TAGS

    def test_sanitize_html_encoding(self):
        html = u'<a href="http://www.ianlewis.org/">テストテスト</a>'.encode("cp932")
        sanitized_html = sanitize_html(html, valid_tags=self.valid_tags, encoding="cp932")
        self.assertEqual(sanitized_html,
            u'<a href="http://www.ianlewis.org/">テストテスト</a>')



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='beproud.utils',
    version='0.34',
    description='Dependency free utilities',
    author='Ian Lewis',
    author_email='ianmlewis@beproud.jp',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=find_packages(),
    namespace_packages=['beproud'],
    test_suite="beproud.utils.tests",
    zip_safe=True,
)



# vim:fileencoding=utf-8
import decimal
import datetime

try:
    import json
except ImportError:
    import simplejson as json

__all__ = (
    'json',
    'SafeJSONEncoder',
    'escapejs_json',
    'force_js',
)

ESCAPEJS_JSON_STRING = (
    (u'<', u'\\u003c'),
    (u'>', u'\\u003e'),
    (u'&', u'\\u0026'),
)

JS_CONVERT_TYPES = {
    'bool': bool,
    'int': int,
    'string': str,
    'array': list,
}


class SafeJSONEncoder(json.JSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date/time and decimal types
    and performs some extra javascript escaping.
    """

    DATE_FORMAT = "%Y-%m-%d"
    TIME_FORMAT = "%H:%M:%S"

    def encode(self, o):
        return escapejs_json(super(SafeJSONEncoder, self).encode(o))

    def default(self, o):
        if isinstance(o, datetime.datetime):
            return o.strftime("%s %s" % (self.DATE_FORMAT, self.TIME_FORMAT))
        elif isinstance(o, datetime.date):
            return o.strftime(self.DATE_FORMAT)
        elif isinstance(o, datetime.time):
            return o.strftime(self.TIME_FORMAT)
        elif isinstance(o, decimal.Decimal):
            return int(o) if o % 1 == 0 else float(str(o))
        else:
            return super(SafeJSONEncoder, self).default(o)


def escapejs_json(s):
    """
    JSONEncoderエスケープされない文字を追加エスケープ
    """
    for c, code in ESCAPEJS_JSON_STRING:
        s = s.replace(c, code)
    return s


def force_js(value, typename=None, encoder=None):
    """
    Changes a python value to javascript for use in templates
    """
    if typename:
        typename = typename.lower()
        if typename in JS_CONVERT_TYPES:
            value = JS_CONVERT_TYPES[typename](value)
    return json.dumps(value, cls=(encoder or SafeJSONEncoder))



# vim:fileencoding=utf-8
import re

from strutils import force_unicode

RE_KEYWORD_EXTRACT = re.compile(u'名詞')
RE_KEYWORD_URL = re.compile(r'(http(s)?:\/\/[A-Za-z0-9%&=~?+-_/.#]+)')
RE_KEYWORD_EXTRACT_IGNORE = re.compile(ur'^[()\-!?"#$%&\'\",.;ー]')
RE_KEYWORD_EXTRACT_IGNORE_KANA = re.compile(ur'^[あ-ん０１２３４５６７８９]+$')
RE_NORMALIZE = re.compile(u'[\u30FC\uFF0D\u2015\u2010]')

__all__ = (
    'extract_keywords',
    'optimize',
    'hankaku',
    'normalize',
    'Token',
    'Tokenizer',
)

def extract_keywords(s, cmpfunc=None, dic_encoding='utf-8'):
    """
    キーワード抽出
    """
    import MeCab
    m = MeCab.Tagger("-Ochasen")
    n = m.parseToNode(RE_KEYWORD_URL.sub(',', optimize(s)).encode(dic_encoding))
    d = {}
    w = u''
    score = 0
    while n:
        features = unicode(n.feature, dic_encoding).split(',')
        # 名詞を抽出
        surface = unicode(n.surface, dic_encoding)
        if RE_KEYWORD_EXTRACT.match(features[0]) and not RE_KEYWORD_EXTRACT_IGNORE.match(surface):
            w += surface
            score += 1
        else:
            if w:
                if len(w) >= 2 and not RE_KEYWORD_EXTRACT_IGNORE_KANA.match(w):
                    if not w in d:
                        d[w] = 0
                    d[w] += 1 #score
                w = u''
                score = 0
        n = n.next
    # ソート
    if cmpfunc:
        cmp_keywords = cmpfunc
    else:
        def cmp_keywords(i1, i2):
            if i1[1] == i2[1]:
                # 長さが同じ場合
                try:
                    if s.index(i1[0]) < s.index(i2[0]):
                        return 1
                except:
                    return -1
                return -1
            return cmp(i1[1], i2[1])

    return sorted(d.items(), cmp=cmp_keywords, reverse=True)

def optimize(s):
    """
    全角記号英数字を半角に、半角かなを全角に
    """
    import zenhan
    s = zenhan.z2h(s, zenhan.ASCII)
    return zenhan.h2z(s, zenhan.KANA)

def hankaku(s):
    import zenhan
    s = zenhan.z2h(s, zenhan.ASCII)
    return zenhan.z2h(s, zenhan.KANA)

def normalize(s):
    """
    全角記号英数字を半角に、半角かなを全角に
    全角数字を半角数字に
    全角の[ー－―‐]を - に
    """
    import zenhan
    s = RE_NORMALIZE.sub('-', s)
    s = zenhan.z2h(s, zenhan.ASCII | zenhan.DIGIT)
    return zenhan.h2z(s, zenhan.KANA)

def token_callback_decorator(callback, min_word_length=1, ignore_word_re=None, word_classes=None):
    def token_callback(word, features):
        def _def_callback(word, features):
            return Token.new(word, "utf8", features=features)

        cb = callback or _def_callback

        token_list = cb(word, features)
        if not isinstance(token_list, (list, tuple)):
            token_list = (token_list,)
        for token in token_list:
            if token is not None and len(token) >= min_word_length and \
                    ((ignore_word_re is None) or not ignore_word_re.match(token)) and \
                    ((word_classes is None) or token.features["word_class"] in word_classes):
                yield token
    return token_callback

class Token(unicode):

    @classmethod
    def new(cls, word, encoding="ascii", features={}):
        token = cls(word, encoding)
        token.features = {
            "word_class": None,
            "sub_word_class1": None,
            "sub_word_class2": None,
            "sub_word_class3": None,
            "conjugated": None, 
            "conjugation": None,
            "base": None,
            "reading": None,
            "pronunciation": None,
        }
        token.features.update(features)
        return token

# MeCab feature マッピング
FEATURE_MAP = (
    "word_class",
    "sub_word_class1",
    "sub_word_class2",
    "sub_word_class3",
    "conjugated",
    "conjugation",
    "base",
    "reading",
    "pronunciation",
)
class Tokenizer(object):
    """
    日本語を汎用的に分割するトーケンアイザー

    token_callbackは word(単語), と特徴辞書を引数として受け取る関数です。
    
    特徴辞書はトーケンの特徴を名前として定義されている辞書です。
    {
        "word_class": 品詞,
        "sub_word_class1": 品詞細分類1,
        "sub_word_class2": 品詞細分類2,
        "sub_word_class3": 品詞細分類3,
        "conjugated": 活用形, (「サ変・スル」,「特殊・ナイ」など
        "conjugation": 活用型, (未然形、基本形など)
        "base": 原形,
        "reading": 読み,
        "pronunciation": 発音,
    }
            
    >>> t = Tokenizer(min_word_length=2, token
    """

    def __init__(self, encoding="utf8", min_word_length=1, ignore_word_re=None, token_callback=None, word_classes=None):
        self.encoding = encoding
        self.min_word_length = min_word_length
        self.ignore_word_re = ignore_word_re
        
        self.token_callback = token_callback_decorator(token_callback, min_word_length, ignore_word_re, word_classes)

    def tokenize(self, obj):
        import MeCab
        tagger = MeCab.Tagger("-Ochasen")
        n = tagger.parseToNode(force_unicode(obj, encoding=self.encoding).encode("utf8"))
        n = n.next

        while n:
            if n.surface:
                features_dict = {}
                for i, feature in enumerate(map(self._clean_feature, n.feature.split(','))):
                    features_dict[FEATURE_MAP[i]] = feature
                
                # 名詞を抽出
                for token in self.token_callback(n.surface, features_dict):
                    yield token 

            n = n.next

    def _clean_feature(self, feature):
        feature = feature.strip()
        return feature if feature != "*" else None
